using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel
{
    public class ObjectPositionSaver : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel"; // edited by VamTastic (original by Frief )
        public string pluginName = "ObjectPositionSaver";
        public string pluginVersion = "1";
        public string pluginDate = "14/01" + "/2023";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    

        private Atom _thisAtom;
        private FreeControllerV3 _thisControl;

        private JSONStorableBool IsFirstLock;
        private JSONStorableBool IsSecondLock;
        private JSONStorableBool IsThirdLock;
        protected JSONStorableStringChooser _firstTargetSelectJSON;
        protected UIDynamicPopup _firstTargetDisplayPopup;
        protected JSONStorableStringChooser _secondTargetSelectJSON;
        protected UIDynamicPopup _secondTargetDisplayPopup;
        protected JSONStorableStringChooser _thirdTargetSelectJSON;
        protected UIDynamicPopup _thirdTargetDisplayPopup;
        protected List<String> _targetObjChoices;

        public Transform FirstGenRoot;
        public Transform SecondGenRoot;
        public Transform ThirdGenRoot;

        protected JSONStorableFloat xPosition;
        protected JSONStorableFloat yPosition;
        protected JSONStorableFloat zPosition;
        protected JSONStorableFloat xRotation;
        protected JSONStorableFloat yRotation;
        protected JSONStorableFloat zRotation;

        protected JSONStorableFloat lockAngleX;
        protected JSONStorableFloat lockAngleY;
        protected JSONStorableFloat lockAngleZ;
        protected JSONStorableFloat lockPosX;
        protected JSONStorableFloat lockPosY;
        protected JSONStorableFloat lockPosZ;

        private Vector3 base_pos;
        private Quaternion base_quaternion;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        public override void Init()
        {
            try
            {
				
                _thisAtom = containingAtom;
                _thisControl = _thisAtom.freeControllers.First(fc => fc.name == "control");

                _targetObjChoices = new List<string>();

                xPosition = new JSONStorableFloat("x position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(xPosition);
                yPosition = new JSONStorableFloat("y position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(yPosition);
                zPosition = new JSONStorableFloat("z position", 0f, OnChangeData, -100f, 100f);
                RegisterFloat(zPosition);

                xRotation = new JSONStorableFloat("x rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(xRotation);
                yRotation = new JSONStorableFloat("y rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(yRotation);
                zRotation = new JSONStorableFloat("z rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(zRotation);

                JSONStorableAction reset_action = new JSONStorableAction("Reset", TriggerSyncReset);
                RegisterAction(reset_action);


                IsFirstLock = new JSONStorableBool("First Lock", false, OnIsFirstLock);
                RegisterBool(IsFirstLock);
                UIDynamicToggle toggle1 = CreateToggle(IsFirstLock, false);
                toggle1.label = "First Lock On Off";

                IsSecondLock = new JSONStorableBool("Second Lock", false, OnIsSecondLock);
                RegisterBool(IsSecondLock);
                UIDynamicToggle toggle2 = CreateToggle(IsSecondLock, false);
                toggle2.label = "Second Lock On Off";

                IsThirdLock = new JSONStorableBool("Third Lock", false, OnIsThirdLock);
                RegisterBool(IsThirdLock);
                UIDynamicToggle toggle3 = CreateToggle(IsThirdLock, false);
                toggle3.label = "Third Lock On Off";

                _firstTargetSelectJSON = new JSONStorableStringChooser("First Choice", null, null, "", OnFirstTargetSelect);
                _firstTargetSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_firstTargetSelectJSON);

                _firstTargetDisplayPopup = CreateScrollablePopup(_firstTargetSelectJSON, false);
                _firstTargetDisplayPopup.popupPanelHeight = 300f;
                _firstTargetDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _firstTargetDisplayPopup.label = "first:";

                _secondTargetSelectJSON = new JSONStorableStringChooser("Second Choice", null, null, "", OnSecondTargetSelect);
                _secondTargetSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_secondTargetSelectJSON);

                _secondTargetDisplayPopup = CreateScrollablePopup(_secondTargetSelectJSON, false);
                _secondTargetDisplayPopup.popupPanelHeight = 300f;
                _secondTargetDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _secondTargetDisplayPopup.label = "second:";

                _thirdTargetSelectJSON = new JSONStorableStringChooser("Third Choice", null, null, "", OnThirdTargetSelect);
                _thirdTargetSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_thirdTargetSelectJSON);

                _thirdTargetDisplayPopup = CreateScrollablePopup(_thirdTargetSelectJSON, false);
                _thirdTargetDisplayPopup.popupPanelHeight = 300f;
                _thirdTargetDisplayPopup.popup.onOpenPopupHandlers += SyncOrificeChoices;
                _thirdTargetDisplayPopup.label = "third:";

                _firstTargetSelectJSON.choices = _targetObjChoices;
                _secondTargetSelectJSON.choices = _targetObjChoices;
                _thirdTargetSelectJSON.choices = _targetObjChoices;

                var btn4 = CreateButton("Target Update", false);

                btn4.button.onClick.AddListener(() =>
                {
                    OnTargetListUpdate();
                });

                UIDynamicButton btn1 = CreateButton("Default Save", false);
                btn1.button.onClick.AddListener(() =>
                {
                    SaveAtomData();
                });

                UIDynamicButton btn2 = CreateButton("Default load", false);
                btn2.button.onClick.AddListener(() =>
                {
                    LoadAtomData();
                });

                lockAngleX = new JSONStorableFloat("lock x rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(lockAngleX);
                CreateSlider(lockAngleX, true);
                lockAngleY = new JSONStorableFloat("lock y rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(lockAngleY);
                CreateSlider(lockAngleY, true);
                lockAngleZ = new JSONStorableFloat("lock z rotation", 0f, OnChangeData, -360f, 360f);
                RegisterFloat(lockAngleZ);
                CreateSlider(lockAngleZ, true);
                lockPosX = new JSONStorableFloat("lock x position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosX);
                CreateSlider(lockPosX, true);
                lockPosY = new JSONStorableFloat("lock y position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosY);
                CreateSlider(lockPosY, true);
                lockPosZ = new JSONStorableFloat("lock z position", 0f, OnChangeData, -1f, 1f);
                RegisterFloat(lockPosZ);
                CreateSlider(lockPosZ, true);


                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Object Position Saver</b></size></color>\n\n" +
                    "<color=#333>" +
                    "store the position of the target,\n" +
                    "or Track a specific target.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("DildoBooth ItemControl  : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

        }

        private bool InitEditor() {
            if (editorInitialized == true) return true;

            // I found my transforms, now the rest
            if (_thisAtom != null)
            {
                editorInitialized = true;

                if (xPosition.val == 0f && yPosition.val == 0f && zPosition.val == 0f) SaveAtomData();

                OnTargetListUpdate();

                if (_targetObjChoices.Count > 0)
                {
                    OnFirstTargetSelect(_firstTargetSelectJSON.val);
                    OnSecondTargetSelect(_secondTargetSelectJSON.val);
                    OnThirdTargetSelect(_thirdTargetSelectJSON.val);
                }
                //SuperController.LogMessage(">> " + _targetObjChoices.Count);

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless

                return true;
            }
            else
            {
                SuperController.LogError("DildoMovement : your is missing mandatory GlassBody.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        void FixedUpdate()
        {
            if (IsFirstLock.val)
            {
                _thisControl.transform.position = FirstGenRoot.position;
                Vector3 offsetPosition = new Vector3(lockPosX.val, lockPosY.val, lockPosZ.val);
                _thisControl.transform.Translate(offsetPosition, _thisControl.transform);
                _thisControl.transform.rotation = FirstGenRoot.rotation;
                _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);

            }
            if (IsSecondLock.val)
            {
                _thisControl.transform.position = SecondGenRoot.position;
                Vector3 offsetPosition = new Vector3(lockPosX.val, lockPosY.val, lockPosZ.val);
                _thisControl.transform.Translate(offsetPosition, _thisControl.transform);
                _thisControl.transform.rotation = SecondGenRoot.rotation;
                _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);
            }
            if (IsThirdLock.val)
            {
                _thisControl.transform.position = ThirdGenRoot.position;
                Vector3 offsetPosition = new Vector3(lockPosX.val, lockPosY.val, lockPosZ.val);
                _thisControl.transform.Translate(offsetPosition, _thisControl.transform);
                _thisControl.transform.rotation = ThirdGenRoot.rotation;
                _thisControl.transform.Rotate(lockAngleX.val, lockAngleY.val, lockAngleZ.val);
            }
        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
        private Transform getObjectRoot( Transform parent, string rootName ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(rootName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child,rootName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

        private void OnIsFirstLock(JSONStorableBool jb)
        {
            if(!jb.val) LoadAtomData();
            else
            {
                IsSecondLock.val = false;
                IsThirdLock.val = false;
            }
        }

        private void OnIsSecondLock(JSONStorableBool jb)
        {
            if (!jb.val) LoadAtomData();
            else
            {
                IsFirstLock.val = false;
                IsThirdLock.val = false;
            }
        }

        private void OnIsThirdLock(JSONStorableBool jb)
        {
            if (!jb.val) LoadAtomData();
            else
            {
                IsFirstLock.val = false;
                IsSecondLock.val = false;
            }
        }

        private void OnFirstTargetSelect(string target_name)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(target_name);
            FirstGenRoot = atom.freeControllers.First(fc => fc.name == "control").transform;
        }

        private void OnSecondTargetSelect(string target_name)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(target_name);
            SecondGenRoot = atom.freeControllers.First(fc => fc.name == "control").transform;
        }

        private void OnThirdTargetSelect(string target_name)
        {
            Atom atom = SuperController.singleton.GetAtomByUid(target_name);
            ThirdGenRoot = atom.freeControllers.First(fc => fc.name == "control").transform;
        }

        protected void SyncOrificeChoices()
        {
            //SuperController.LogMessage(">> SyncOrificeChoices ");
            //UpdateOrificeChoices();
        }

        private void OnTargetListUpdate()
        {
            List<string> uids = SuperController.singleton.GetAtomUIDs();

            _targetObjChoices = new List<string>();

            foreach (string uid in uids)
            {
                //Atom atom = SuperController.singleton.GetAtomByUid(uid);
                _targetObjChoices.Add(uid);
            }

            _firstTargetSelectJSON.choices = _targetObjChoices;
            _secondTargetSelectJSON.choices = _targetObjChoices;
            _thirdTargetSelectJSON.choices = _targetObjChoices;
        }

        private void SaveAtomData()
        {
            Vector3 base_pos = _thisControl.transform.position;
            Vector3 base_angle = _thisControl.transform.eulerAngles;

            xPosition.val = base_pos.x;
            yPosition.val = base_pos.y;
            zPosition.val = base_pos.z;

            xRotation.val = base_angle.x;
            yRotation.val = base_angle.y;
            zRotation.val = base_angle.z;

            
        }

        private void LoadAtomData()
        {
            _thisControl.transform.position = new Vector3(xPosition.val, yPosition.val, zPosition.val);
            _thisControl.transform.rotation = Quaternion.Euler(new Vector3(xRotation.val, yRotation.val, zRotation.val));

        }

        void OnChangeData(float value)
        {
            
        }

        private void TriggerSyncReset()
        {
            IsFirstLock.val = false;
            IsSecondLock.val = false;
            IsThirdLock.val = false;
            LoadAtomData();
        }
    }
	
}